/*
 *  Copyright (C) Chaperon. All rights reserved.
 *  -------------------------------------------------------------------------
 *  This software is published under the terms of the Apache Software License
 *  version 1.1, a copy of which has been included  with this distribution in
 *  the LICENSE file.
 */

package net.sourceforge.chaperon.grammar.token;

import java.io.Serializable;

import net.sourceforge.chaperon.grammar.Grammar;
import net.sourceforge.chaperon.grammar.symbol.TerminalSymbol;
import net.sourceforge.chaperon.grammar.token.definition.CharacterSequence;
import net.sourceforge.chaperon.grammar.token.definition.Alternation;
import net.sourceforge.chaperon.grammar.token.definition.BeginOfLine;
import net.sourceforge.chaperon.grammar.token.definition.CharacterClass;
import net.sourceforge.chaperon.grammar.token.definition.CharacterClassElement;
import net.sourceforge.chaperon.grammar.token.definition.CharacterSequence;
import net.sourceforge.chaperon.grammar.token.definition.Concatenation;
import net.sourceforge.chaperon.grammar.token.definition.DefinitionElement;
import net.sourceforge.chaperon.grammar.token.definition.EndOfLine;
import net.sourceforge.chaperon.grammar.token.definition.UniversalCharacter;

/**
 * This class represents a token. Like<br/>
 * identifier := [A-Za-z][A-Za-z0-9_]
 *
 * @author Stephan Michels
 * @version $Revision: 1.7 $ $Date: 2002/05/06 16:22:56 $
 */
public class Token implements Serializable, Cloneable
{

	private TokenList         _owner = null;

	private TerminalSymbol    _tsymbol = TerminalSymbol.valueOf("noname");
	private DefinitionElement _definition = new CharacterSequence("");

	private Associativity     _associativity = Associativity.NONASSOC;

	private int               _priority = -1;

	/**
	 * Creates a token
	 */
	public Token()
	{
	}

	/**
	 * Creates a token
	 *
	 * @param tsymbol The terminal symbol for this token
	 */
	public Token(TerminalSymbol tsymbol)
	{
		setSymbol(tsymbol);
	}

	/**
	 * Creates a token
	 *
	 * @param tsymbol The terminal symbol for this token
	 * @param definition The regex definition defined through
	 *                   a TokenElement
	 */
	public Token(TerminalSymbol tsymbol, DefinitionElement definition)
	{
		setSymbol(tsymbol);
		setDefinition(definition);
	}

	/**
	 *
	 *
	 * @param owner      
	 */
	public void setOwner(TokenList owner)
	{
		if (_owner != null)
			throw new IllegalStateException("Token has already a owner");
		_owner = owner;
	}

	/**
	 *
	 *
	 * @return
	 */
	public TokenList getOwner()
	{
		return _owner;
	}

	/**
	 * Sets the symbol for this token
	 *
	 * @param tsymbol Terminal symbol
	 */
	public void setSymbol(TerminalSymbol tsymbol)
	{
		if (tsymbol == null)
			throw new NullPointerException();
		_tsymbol = tsymbol;
	}

	/**
	 * Returns the symbol from this token
	 *
	 * @return Terminal symbol
	 */
	public TerminalSymbol getSymbol()
	{
		return _tsymbol;
	}

	/**
	 * Sets the definition for this token
	 *
	 * @param definition The regex definition defined through
	 *                   a TokenElement
	 */
	public void setDefinition(DefinitionElement definition)
	{
		_definition = definition;
	}

	/**
	 * Returns the definition
	 *
	 * @return The regex definition defined through
	 *         a TokenElement
	 */
	public DefinitionElement getDefinition()
	{
		return _definition;
	}

	/**
	 * Sets the associativity od the token
	 *
	 * @param associativity NONASSOC, LEFT or RIGHT
	 */
	public void setAssociativity(Associativity associativity)
	{
		_associativity = associativity;
	}

	/**
	 * Returns the associativity
	 *
	 * @return NONASSOC, LEFT or RIGHT
	 */
	public Associativity getAssociativity()
	{
		return _associativity;
	}

	/**
	 * Sets the priority the the token.
	 * (Higher is better)
	 *
	 * @param priority Priority of the token
	 */
	public void setPriority(int priority)
	{
		if (priority >= 0)
			_priority = priority;
	}

	/**
	 * Returns the priority of the token
	 *
	 * @return Priority of the token
	 */
	public int getPriority()
	{
		if (_priority >= 0)
			return _priority;
		return _owner.getTokenCount() - _owner.indexOf(this);
	}

	/**
	 * Convert the token to a string
	 *
	 * @return Content of the token
	 */
	public String toString()
	{
		StringBuffer buffer = new StringBuffer();

		buffer.append(_tsymbol.getName());
		buffer.append(" := \"");
		buffer.append(_definition.toString());
		buffer.append("\"[Prior=");
		buffer.append(String.valueOf(getPriority()));
		buffer.append("] [assoc=");
		buffer.append(_associativity);
		buffer.append("]");

		return buffer.toString();
	}

	/**
	 *
	 *
	 * @return
	 *
	 * @throws CloneNotSupportedException
	 */
	public Object clone() throws CloneNotSupportedException
	{
		Token clone = new Token();

		clone.setSymbol(_tsymbol);
		clone.setDefinition((DefinitionElement)_definition.clone());
		if (_priority >= 0)
			clone.setPriority(_priority);
		clone.setAssociativity(_associativity);

		return clone;
	}
}
